'''
graphing package contains functions to easily process and graph dataframes into conveniently formatted visualizations
'''

from .visualizations import graphing_ready_dataframe
from .visualizations import visualization

